#include "eldbus_elementary_colorclass.h"

static int _log_main = -1;
#undef ERR
#define ERR(...) EINA_LOG_DOM_ERR(_log_main, __VA_ARGS__);
int ELEMENTARY_COLORCLASS_EDIT_EVENT = 0;
int ELEMENTARY_COLORCLASS_CHANGED_EVENT = 0;
int ELEMENTARY_COLORCLASS_RESET_EVENT = 0;

void 
elementary_colorclass_send_c_c_call(Eldbus_Proxy *proxy, Eina_Value *args)
{
   EINA_SAFETY_ON_NULL_RETURN(proxy);
   Eldbus_Message *msg = eldbus_proxy_method_call_new(proxy, "SendCC");
   if (!eldbus_message_from_eina_value("tba(ssa(iiii))", msg, args))
     {
        ERR("Error: Filling message from eina value.");
        eldbus_message_unref(msg);
        return;
     }
   eldbus_proxy_send(proxy, msg, NULL, NULL, -1);
}

void
elementary_colorclass_close_call(Eldbus_Proxy *proxy, uint64_t winid)
{
   Eldbus_Message *msg;
   EINA_SAFETY_ON_NULL_RETURN(proxy);
   msg = eldbus_proxy_method_call_new(proxy, "Close");
   if (!eldbus_message_arguments_append(msg, "t", winid))
     {
        ERR("Error: Filling message.");
        eldbus_message_unref(msg);
        return;
     }
   eldbus_proxy_send(proxy, msg, NULL, NULL, -1);
}

static void
elementary_colorclass_edit_data_free(void *user_data EINA_UNUSED, void *func_data)
{
   Elementary_Colorclass_Edit_Data *s_data = func_data;
   free(s_data);
}

static void
on_elementary_colorclass_edit(void *data, const Eldbus_Message *msg)
{
   Eldbus_Proxy *proxy = data;
   Elementary_Colorclass_Edit_Data *s_data = calloc(1, sizeof(Elementary_Colorclass_Edit_Data));
   s_data->proxy = proxy;
   if (!eldbus_message_arguments_get(msg, "t", &s_data->winid))
     {
        ERR("Error: Getting arguments from message.");
        free(s_data);
        return;
     }
   ecore_event_add(ELEMENTARY_COLORCLASS_EDIT_EVENT, s_data, elementary_colorclass_edit_data_free, NULL);
}

static void
elementary_colorclass_changed_data_free(void *user_data EINA_UNUSED, void *func_data)
{
   Elementary_Colorclass_Changed_Data *s_data = func_data;
   eina_value_free(s_data->value);
   free(s_data);
}

static void
on_elementary_colorclass_changed(void *data, const Eldbus_Message *msg)
{
   Eldbus_Proxy *proxy = data;
   Elementary_Colorclass_Changed_Data *s_data = calloc(1, sizeof(Elementary_Colorclass_Changed_Data));
   s_data->proxy = proxy;
   s_data->value = eldbus_message_to_eina_value(msg);
   ecore_event_add(ELEMENTARY_COLORCLASS_CHANGED_EVENT, s_data, elementary_colorclass_changed_data_free, NULL);
}

static void
elementary_colorclass_reset_data_free(void *user_data EINA_UNUSED, void *func_data)
{
   Elementary_Colorclass_Reset_Data *s_data = func_data;
   free(s_data->name);
   free(s_data);
}

static void
on_elementary_colorclass_reset(void *data, const Eldbus_Message *msg)
{
   Eldbus_Proxy *proxy = data;
   Elementary_Colorclass_Reset_Data *s_data = calloc(1, sizeof(Elementary_Colorclass_Reset_Data));
   s_data->proxy = proxy;
   if (!eldbus_message_arguments_get(msg, "ts", &s_data->winid, &s_data->name))
     {
        ERR("Error: Getting arguments from message.");
        free(s_data);
        return;
     }
   s_data->name = strdup(s_data->name);
   ecore_event_add(ELEMENTARY_COLORCLASS_RESET_EVENT, s_data, elementary_colorclass_reset_data_free, NULL);
}

void
elementary_colorclass_log_domain_set(int id)
{
   _log_main = id;
}

void
elementary_colorclass_proxy_unref(Eldbus_Proxy *proxy)
{
   Eldbus_Object *obj = eldbus_proxy_object_get(proxy);
   eldbus_proxy_unref(proxy);
   eldbus_object_unref(obj);
}

Eldbus_Proxy *
elementary_colorclass_proxy_get(Eldbus_Connection *conn, const char *bus, const char *path)
{
   Eldbus_Object *obj;
   Eldbus_Proxy *proxy;
   EINA_SAFETY_ON_NULL_RETURN_VAL(conn, NULL);
   EINA_SAFETY_ON_NULL_RETURN_VAL(bus, NULL);
   if (!path) path = "/";
   obj = eldbus_object_get(conn, bus, path);
   proxy = eldbus_proxy_get(obj, "org.elementary.colorclass");
   eldbus_proxy_signal_handler_add(proxy, "Edit", on_elementary_colorclass_edit, proxy);
   if (!ELEMENTARY_COLORCLASS_EDIT_EVENT)
     ELEMENTARY_COLORCLASS_EDIT_EVENT = ecore_event_type_new();
   eldbus_proxy_signal_handler_add(proxy, "Changed", on_elementary_colorclass_changed, proxy);
   if (!ELEMENTARY_COLORCLASS_CHANGED_EVENT)
     ELEMENTARY_COLORCLASS_CHANGED_EVENT = ecore_event_type_new();
   eldbus_proxy_signal_handler_add(proxy, "Reset", on_elementary_colorclass_reset, proxy);
   if (!ELEMENTARY_COLORCLASS_RESET_EVENT)
     ELEMENTARY_COLORCLASS_RESET_EVENT = ecore_event_type_new();
   return proxy;
}
